#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: http://www.narda-sts.de
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import wx
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import MeasDlgThread
# Import the dialog event class
from .gui.mydialogs import MyMeasGUIDlg, MySettingsDlg


class MyMeasurement(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = 'Examples'
        self._scr_title = 'Examples03 - Maximum Value Measurement'
        self._scr_description = 'Starts the measurement, displays value of each measurement in a dialog, ' \
                                'and outputs the maximum value. '
        # self._icon_path = self.script_path.joinpath('NardaScriptLauncher_example01.png')
        self._list_prio = 3
        self._nsl_executed_behavior = NSL_Executed_Behaviors.SHOW_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        # self.my_variable = None

    def _run_script(self, args):
        """ Script main function

        This method is called when a user clicks on the corresponding script button.

        Use the dialog classes only in this part of the code.
        """

        # Optional: Prepare some parameters for the measurement.
        measurement_steps = 10

        # Instantiate dialog event class.
        # -------------------------------------------------------------------------------
        meas_dlg = MyMeasGUIDlg(self.main_gui, 'Title')

        # Create an additional measurement thread to keep the GUI alive.
        # Please change 'MyUserScript._run_measurement' according to your class name.
        # With 'args=[param1, parm2, paramn]' you can pass some optional parameters to the measurement thread.
        mthread = MeasDlgThread(main_gui=self.main_gui, dlg=meas_dlg,
                                callback=MyMeasurement._run_measurement,
                                args=[self.signalshark.addr, measurement_steps])

        # Start the measurement thread and show a visualization (progress) dialog.
        mthread.start_measurement()
        # The lines of code below will be executed after the measurement thread has done it's job.
        # ...

        # Evaluate the result of the measurement thread (if any).
        if mthread.result is not None:
            self.MessageBoxModal('Max value: ' + str(mthread.result),
                                 'Measurement Result', wx.OK | wx.ICON_INFORMATION)

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         signalshark_addr: str, measurement_steps: int):
        """ Main method of the measurement thread that handles a time consuming measurement.

        Please do NOT use any wxPython elements like a MessageBox or a Dialog here!
        They are only permitted in the GUI thread ('_run_script' method).
        Please use the 'update_progress' method to display information!

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param signalshark_addr: Custom parameter to handover the ip address of the SignalShark
        :param measurement_steps: Custom parameter
        :return: Optional custom parameter
        """

        # Initialize return value (if any)
        max_value = -999.0

        # Setup connection to the SignalShark and display error message if connection fails.
        signalshark = SignalSharkDev(signalshark_addr)
        if not signalshark.connect():
            return wait_for_stopevent(msg='Cannot connect to the device!', icon_style=wx.ICON_ERROR)  # Returns None

        # Clear SCPI error queue
        signalshark.scpi.check_error()

        try:
            # Prepare progress value calculation. Progress is a value from 0.0 to 1.0
            progress_step = 1.0 / measurement_steps
            act_progress = 0.0

            # Setup measurement:
            # Check if desired task exists.
            # If task does not exist, add one and do initialization stuff.
            if signalshark.scpi.check_add_task(TaskTypes.RT_SPECTRUM, 'MyMeas'):
                pass

            # Check if desired view exists and add one if it does not.
            exists, index = signalshark.scpi.check_add_view(ViewTypes.LEVEL)
            if not exists:
                # return wait_for_stopevent('Please setup the newly added Level Meter View.')
                return wait_for_stopevent('Level Meter View was added to the task. '
                                          'Please setup Level Meter View and restart the script.')

            # Update the dialog message text and show a progress bar
            update_progress(msg='Starting measurement', show_progress=True)

            # Loop, that simulates a time consuming measurement
            scan_number = -1
            last_scan_number = -1
            loop_cnt = 0
            while loop_cnt < measurement_steps:
                if stopevent.isSet():
                    return None

                # Example measurement:
                # Wait until Scan Number has increased.
                scan_number = signalshark.scpi.levelmeter.get_data_update(last_scan_number)
                if scan_number < 0:
                    return wait_for_stopevent('Cannot acquire measurement data!', icon_style=wx.ICON_ERROR)
                elif scan_number > last_scan_number:
                    last_scan_number = scan_number
                else:
                    stopevent.wait(0.5)
                    continue

                # Update maximum value if determined level meter value is greater.
                det_val, det_minmax = signalshark.scpi.levelmeter.get_data_detector(1)
                if det_val > max_value:
                    max_value = det_val

                # Update the progress bar.
                act_progress += progress_step
                update_progress(progress=act_progress, data=det_val)
                loop_cnt += 1

                # Allow the thread to synchronize the stop event.
                stopevent.wait(0.5)

            # Return some optional measurement result (number, string, list, dict, object ...)
            return max_value
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e), icon_style=wx.ICON_ERROR)  # Returns None
        finally:
            # Close connection.
            signalshark.disconnect()

